### -*- coding: utf-8 -*- #############################################
# Разработано компанией Ключевые Решения (http://keysolutions.ru/)
# Все права защищены, 2006-2007
#
# Developed by Key Solutions (http://keysolutions.ru/)
# All right reserved, 2006-2007
#######################################################################
"""Captcha class for the Zope 3 based ks.schema.captcha package

$Id: captcha.py 35330 2008-01-13 09:03:40Z cray $
"""

__author__  = "Anatoly Bubenkov"
__license__ = "ZPL"
__version__ = "$Revision: 35330 $"
__date__ = "$Date: 2008-01-13 12:03:40 +0300 (Вск, 13 Янв 2008) $"
__credits__ = "Based heavily on zope.app.form.browser.objectwidget.ObjectWidget"

from zope.schema import Field
from zope.schema.interfaces import RequiredMissing
from interfaces import ICaptcha, CodeIsInvalidError
from zope.interface import implements
from zope.interface.interfaces import IInterface
import ks.captcha.interfaces
from zope.component import getSiteManager, getUtility
import random
from zope.security import checkPermission
from ks.schema.notset.notset import NotSetMixin
from ks.schema.captcha import _

class Captcha(NotSetMixin, Field):

    implements(ICaptcha)

    def __init__(self, captcha=None, **kw):
        """Initialize object."""
        if captcha is not None:
            assert (isinstance(captcha, basestring)
                    or ks.captcha.interfaces.ICaptcha.providedBy(captcha))

        self.captcha = None
        self.captchaName = ''
        if isinstance(captcha, (unicode, str)):
            self.captchaName = captcha
        else:
            self.captcha = captcha
        self._init_field = bool(self.captchaName)
        super(Captcha, self).__init__(**kw)
        self._init_field = False

    def bind(self, object):
        """See zope.schema._bootstrapinterfaces.IField."""
        clone = super(Captcha, self).bind(object)
        # get registered vocabulary if needed:
        if ks.captcha.interfaces.ICaptcha.providedBy(self.captcha):
            clone.captcha = self.captcha
            assert ks.captcha.interfaces.ICaptcha.providedBy(clone.captcha)
        elif clone.captcha is None:
            clone.captcha = getUtility(ks.captcha.interfaces.ICaptcha, name=self.captchaName, context=object)
            assert ks.captcha.interfaces.ICaptcha.providedBy(clone.captcha)

        return clone

    def _validate(self, value):
        # Pass all validations during initialization
        if self._init_field:
            return
        super(Captcha, self)._validate(value)
        captcha = self.captcha
        if captcha is None:
            try:
                captcha = getUtility(ks.captcha.interfaces.ICaptcha, name=self.captchaName, context=self.context)
            except TypeError:
                raise ValueError("Can't validate value without image checker")
        if not checkPermission("ks.schema.captcha.canPassCaptcha", self.context):
            if value[1] is None:
                raise RequiredMissing
            if not captcha.check(*value):
                raise CodeIsInvalidError(value)



### -*- coding: utf-8 -*- #############################################
# Разработано компанией Ключевые Решения (http://keysolutions.ru/)
# Все права защищены, 2006-2007
#
# Developed by Key Solutions (http://keysolutions.ru/)
# All right reserved, 2006-2007
#######################################################################
"""ObjectDisplayWidget class for the Zope 3 based ks.widget package

$Id: objectdisplaywidget.py 35330 2008-01-13 09:03:40Z cray $
"""

__author__  = "Anatoly Zaretsky"
__license__ = "ZPL"
__version__ = "$Revision: 35330 $"
__date__ = "$Date: 2008-01-13 12:03:40 +0300 (Вск, 13 Янв 2008) $"
__credits__ = "Based heavily on zope.app.form.browser.objectwidget.ObjectWidget"

from zope.interface import implements
from zope.schema import getFieldNamesInOrder

from zope.app.form.interfaces import IDisplayWidget
from zope.app.form.utility import setUpWidgets
from zope.app.form.browser.widget import DisplayWidget
from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile

# from ks.widget import _ не используется

class ObjectDisplayWidgetView(object):

    template = ViewPageTemplateFile('objectdisplaywidget.pt')

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        return self.template()


class ObjectDisplayWidget(DisplayWidget):

    implements (IDisplayWidget)

    names = None

    viewFactory = ObjectDisplayWidgetView

    def __init__(self, context, request, **kw):
        super(ObjectDisplayWidget, self).__init__(context, request)

        # define view that renders the widget
        self.view = self.viewFactory(self, request)

        # handle foo_widget specs being passed in
        if self.names is None:
            self.names = getFieldNamesInOrder(self.context.schema)
        for k, v in kw.items():
            if k.endswith('_widget'):
                setattr(self, k, v)

        # set up my subwidgets
        self._setUpWidgets()


    def setPrefix(self, prefix):
        super(ObjectDisplayWidget, self).setPrefix(prefix)
        # XXX: is this really needed for display widgets?
        self._setUpWidgets()

    def _setUpWidgets(self):
        # subwidgets need a new name
        setUpWidgets(self, self.context.schema, IDisplayWidget,
                         prefix=self.name, names=self.names,
                         context=self.context)

    def __call__(self):
        return self.view()

    def legendTitle(self):
        return self.context.title or self.context.__name__

    def getSubWidget(self, name):
        return getattr(self, '%s_widget' % name)

    def subwidgets(self):
        return [self.getSubWidget(name) for name in self.names]

    def hidden(self):
        """Render the object as hidden fields."""
        result = []
        for name in self.names:
            result.append(getSubwidget(name).hidden())
        return "".join(result)

    def setRenderedValue(self, value):
        """Set the default data for the widget.

        The given value should be used even if the user has entered
        data.
        """
        # re-call setupwidgets with the content
        self._setUpWidgets()
        for name in self.names:
            self.getSubWidget(name).setRenderedValue(getattr(value, name, None))



### -*- coding: utf-8 -*- #############################################
# Разработано компанией Ключевые Решения (http://keysolutions.ru/)
# Все права защищены, 2006-2007
#
# Developed by Key Solutions (http://keysolutions.ru/)
# All right reserved, 2006-2007
#######################################################################
"""The widget regular expression classes.

$Id: rewidget.py 35330 2008-01-13 09:03:40Z cray $
"""
__author__  = "Arvid"
__license__ = "ZPL"
__version__ = "$Revision: 35330 $"
__date__ = "$Date: 2008-01-13 12:03:40 +0300 (Вск, 13 Янв 2008) $"

from zope.schema.interfaces import ValidationError
from zope.app.form.browser.textwidgets import TextWidget, PasswordWidget
from zope.app.form.interfaces import WidgetInputError
from zope.app.form.browser.widget import SimpleInputWidget
from zope.schema.interfaces import InvalidValue
from ks.schema.textline import _

import re

dot_atom = r'[-A-Za-z0-9!#$%&\'*+/=?^_{|}~]+'
domain = r'[-A-Za-z0-9]'

emailConstraint = re.compile(r'^%(dot_atom)s(?:\.%(dot_atom)s)*@%(domain)s+(?:\.%(domain)s+)*(?:\.%(domain)s{2,})+$' % {'dot_atom': dot_atom, 'domain' : domain})
nameConstraint = re.compile(u'^[a-zA-Z0-9_-]{2,}$', flags=re.U)

class NotValidName(ValidationError):
    __doc__ = _(u"""Alias must be min 2 characters length and consist of a-z A-Z, -, _ and 0-9""")

class NotValidEmail(ValidationError):
    __doc__ = _(u"""Email must be in format:
        local part (symbols, separated by .),
        sign @, domain (characters, 0-9, -,
        separated by ., and length of first-level domain 2 characters min).
        symbols - это latin characters, numbers,
        !, #, $, %, &, ', *, +, -, /, =, ?, ^, _, {, |, }, ~
    """)

class WidgetREBase(TextWidget):
    """widget re checker"""
    reObj = None
    exceptionRE = ValidationError

    def getInputValue(self):
        value = super(WidgetREBase, self).getInputValue()

        if value and not (hasattr(self, '_error') and self._error):
            try:
                if self.reObj.match(value) is None:
                    raise self.exceptionRE
            except self.exceptionRE, v:
                self._error = WidgetInputError(self.context.__name__, self.label, v)
                raise self._error

        return value

class EmailWidget(WidgetREBase):
    """e-mail widget"""
    reObj = emailConstraint
    exceptionRE = NotValidEmail

class NameWidget(WidgetREBase):
    """name widget"""
    reObj = nameConstraint
    exceptionRE = NotValidName



### -*- coding: utf-8 -*- #############################################
# Разработано компанией Ключевые Решения (http://keysolutions.ru/)
# Все права защищены, 2006-2007
#
# Developed by Key Solutions (http://keysolutions.ru/)
# All right reserved, 2006-2007
#######################################################################
"""SmartBytes class for the Zope 3 based ks.schema.smartbytes package

$Id: smartbytes.py 35330 2008-01-13 09:03:40Z cray $
"""

__author__  = "Anatoly Bubenkov"
__license__ = "ZPL"
__version__ = "$Revision: 35330 $"
__date__ = "$Date: 2008-01-13 12:03:40 +0300 (Вск, 13 Янв 2008) $"
__credits__ = "Based heavily on zope.app.form.browser.objectwidget.ObjectWidget"

from zope.schema import Bytes

class SmartBytes(Bytes):
    """Smart Bytes"""

    def set(self, object, value):
        if value is None or not value:
            return
        return super(SmartBytes, self).set(object, value)



### -*- coding: utf-8 -*- #############################################
# Разработано компанией Ключевые Решения (http://keysolutions.ru/)
# Все права защищены, 2006-2007
#
# Developed by Key Solutions (http://keysolutions.ru/)
# All right reserved, 2006-2007
#######################################################################
"""Inrefaces for the Zope 3 based ks.schema.captcha.demo package

$Id: interfaces.py 35330 2008-01-13 09:03:40Z cray $
"""
__author__  = "Anatoly Bubenkov"
__license__ = "ZPL"
__version__ = "$Revision: 35330 $"
__date__ = "$Date: 2008-01-13 12:03:40 +0300 (Вск, 13 Янв 2008) $"

from ks.schema import _, Captcha
from zope.interface import Interface

class ICaptchaDemo(Interface):

    captcha = Captcha(title=_(u'Captcha'))



### -*- coding: utf-8 -*- #############################################
# Разработано компанией Ключевые Решения (http://keysolutions.ru/)
# Все права защищены, 2006-2007
#
# Developed by Key Solutions (http://keysolutions.ru/)
# All right reserved, 2006-2007
#######################################################################
"""NotSetMixin class for the Zope 3 based ks.schema.notset package

$Id: notset.py 35330 2008-01-13 09:03:40Z cray $
"""

__author__  = "Anatoly Bubenkov"
__license__ = "ZPL"
__version__ = "$Revision: 35330 $"
__date__ = "$Date: 2008-01-13 12:03:40 +0300 (Вск, 13 Янв 2008) $"
__credits__ = "Based heavily on zope.app.form.browser.objectwidget.ObjectWidget"

from zope.schema import Bytes, Bool

class NotSetMixin(object):

    def set(self, object, value):
        if self.readonly:
            raise TypeError("Can't set values on read-only fields "
                            "(name=%s, class=%s.%s)"
                            % (self.__name__,
                               object.__class__.__module__,
                               object.__class__.__name__))

class BytesNotSet(NotSetMixin, Bytes):
    """Bytes Not Set"""

class BoolNotSet(NotSetMixin, Bool):
    """Bool Not Set"""



### -*- coding: utf-8 -*- #############################################
# Разработано компанией Ключевые Решения (http://keysolutions.ru/)
# Все права защищены, 2006-2007
#
# Developed by Key Solutions (http://keysolutions.ru/)
# All right reserved, 2006-2007
#######################################################################
"""SelectDateWidget class for the Zope 3 based ks.widget package

$Id: selectdatewidget.py 35330 2008-01-13 09:03:40Z cray $
"""

__author__  = "Anatoly Zaretsky"
__license__ = "ZPL"
__version__ = "$Revision: 35330 $"
__date__ = "$Date: 2008-01-13 12:03:40 +0300 (Вск, 13 Янв 2008) $"

import datetime

from zope.app.form.interfaces import ConversionError, InputErrors
from zope.app.form.browser.widget import SimpleInputWidget, renderElement

from ks.schema.date import _

class SelectDateWidget(SimpleInputWidget):

    tag = u'select'
    cssClass = u''
    extra = u''
    _missing = u''

    minYear = None
    maxYear = None

    minYearDelta = 100
    maxYearDelta = 0

    @property
    def _day_name(self):
        return self.name + '.day'

    @property
    def _month_name(self):
        return self.name + '.month'

    @property
    def _year_name(self):
        return self.name + '.year'

    def hasInput(self):
        return \
            self._day_name in self.request.form and \
            self._month_name in self.request.form and \
            self._year_name in self.request.form

    def _hasPartialInput(self):
        return \
            self._day_name in self.request.form or \
            self._month_name in self.request.form or \
            self._year_name in self.request.form

    def _getFieldInput(self, name):
        return self.request.form.get(name, self._missing)

    def _getFormInput(self):
        return (\
            self._getFieldInput(self._day_name),
            self._getFieldInput(self._month_name),
            self._getFieldInput(self._year_name))

    def _toFieldValue(self, (day, month, year)):
        if day == self._missing or month == self._missing or year == self._missing:
            return self.context.missing_value
        else:
            try:
                return datetime.date(int(year), int(month), int(day))
            except ValueError, e:
                raise ConversionError(_(u"Incorrect string data for date"), e)

    def _toFormValue(self, value):
        if value == self.context.missing_value:
            d = datetime.date.today()
            return (d.day, d.month, d.year)
        else:
            return (value.day, value.month, value.year)

    def _getFormValue(self):
        if not self._renderedValueSet():
            if self._hasPartialInput():
                error = self._error
                try:
                    try:
                        value = self.getInputValue()
                    except InputErrors:
                        return self._getFormInput()
                finally:
                    self._error = error
            else:
                value = self._getDefault()
        else:
            value = self._data
        return self._toFormValue(value)

    def _options(self, current, min, max, render=lambda x: x):
        if current != self._missing:
            try:
                current = int(current)
            except ValueError:
                current = self._missing
        if current == self._missing:
            current = min
        options = []
        for i in xrange(min, max + 1):
            if i == current:
                o = renderElement('option', value=i, contents=render(i), selected="selected")
            else:
                o = renderElement('option', value=i, contents=render(i))
            options.append(o)
        return ''.join(options)

    def _days(self, day):
        return self._options(day, 1, 31)

    def _months(self, month):
        names = self.request.locale.dates.calendars[u'gregorian'].getMonthNames()
        return self._options(month, 1, 12, lambda m: names[m - 1])

    def _years(self, year):
        minYear = self.minYear
        if minYear is None:
            minYear = datetime.date.today().year - int(self.minYearDelta)
        maxYear = self.maxYear
        if maxYear is None:
            maxYear = datetime.date.today().year + int(self.maxYearDelta)
        return self._options(year, minYear, maxYear)

    def __call__(self):
        value = self._getFormValue()
        ds = renderElement('select',
                           name=self._day_name,
                           id=self._day_name,
                           cssClass=self.cssClass,
                           extra=self.extra,
                           contents=self._days(value[0]))
        ms = renderElement('select',
                           name=self._month_name,
                           id=self._month_name,
                           cssClass=self.cssClass,
                           extra=self.extra,
                           contents=self._months(value[1]))
        ys = renderElement('select',
                           name=self._year_name,
                           id=self._year_name,
                           cssClass=self.cssClass,
                           extra=self.extra,
                           contents=self._years(value[2]))
        return ''.join((ds, ms, ys))

    def hidden(self):
        value = self._getFormValue()
        ds = renderElement('input',
                           name=self._day_name,
                           id=self._day_name,
                           type='hidden',
                           value=value[0],
                           extra=self.extra)
        ms = renderElement('input',
                           name=self._month_name,
                           id=self._month_name,
                           type='hidden',
                           value=value[1],
                           extra=self.extra)
        ys = renderElement('input',
                           name=self._year_name,
                           id=self._year_name,
                           type='hidden',
                           value=value[2],
                           extra=self.extra)
        return ''.join((ds, ms, ys))

class SelectDatetimeWidget(SelectDateWidget):

    @property
    def _hour_name(self):
        return self.name + '.hour'

    @property
    def _minute_name(self):
        return self.name + '.minute'

    @property
    def _second_name(self):
        return self.name + '.second'

    def hasInput(self):
        return super(SelectDatetimeWidget, self).hasInput() and \
                    self._hour_name in self.request.form and \
                    self._minute_name in self.request.form and \
                    self._second_name in self.request.form

    def _hasPartialInput(self):
        return super(SelectDatetimeWidget, self)._hasPartialInput() or \
            self._hour_name in self.request.form or \
            self._minute_name in self.request.form or \
            self._second_name in self.request.form

    def _getFormInput(self):
        return super(SelectDatetimeWidget, self)._getFormInput() +\
                    (self._getFieldInput(self._hour_name),
                    self._getFieldInput(self._minute_name),
                    self._getFieldInput(self._second_name))

    def _hours(self, hour):
        return self._options(hour, 0, 23)

    def _minutes(self, minute):
        return self._options(minute, 0, 59)

    def _seconds(self, second):
        return self._options(second, 0, 59)

    def _toFormValue(self, value):
        if value == self.context.missing_value:
            d = datetime.datetime.now()
            return (d.day, d.month, d.year, d.hour, d.minute, d.second)
        else:
            return (value.day, value.month, value.year, value.hour, value.minute, value.second)

    def _toFieldValue(self, (day, month, year, hour, minute, second)):
        if day == self._missing \
           or month == self._missing \
           or year == self._missing \
           or hour == self._missing \
           or minute == self._missing \
           or second == self._missing:
            return self.context.missing_value
        else:
            try:
                return datetime.datetime(int(year), int(month), int(day), int(hour), int(minute), int(second))
            except ValueError, e:
                raise ConversionError(_(u"Incorrect string data for date and time"), e)

    def __call__(self):
        res = super(SelectDatetimeWidget, self).__call__()
        value = self._getFormValue()
        return ''.join((res,
                        renderElement('select',
                           name=self._hour_name,
                           id=self._hour_name,
                           cssClass=self.cssClass,
                           extra=self.extra,
                           contents=self._hours(value[3])),
        renderElement('select',
                           name=self._minute_name,
                           id=self._minute_name,
                           cssClass=self.cssClass,
                           extra=self.extra,
                           contents=self._minutes(value[4])),
        renderElement('select',
                           name=self._second_name,
                           id=self._second_name,
                           cssClass=self.cssClass,
                           extra=self.extra,
                           contents=self._seconds(value[5]))))

    def hidden(self):
        res = super(SelectDatetimeWidget, self).hidden()
        value = self._getFormValue()
        return ''.join((res,
                        renderElement('input',
                           name=self._hour_name,
                           id=self._hour_name,
                           type='hidden',
                           value=value[3],
                           extra=self.extra),
        renderElement('input',
                           name=self._minute_name,
                           id=self._minute_name,
                           type='hidden',
                           value=value[4],
                           extra=self.extra),
        renderElement('input',
                           name=self._second_name,
                           id=self._second_name,
                           type='hidden',
                           value=value[5],
                           extra=self.extra)))



### -*- coding: utf-8 -*- #############################################
# Разработано компанией Ключевые Решения (http://keysolutions.ru/)
# Все права защищены, 2006-2007
#
# Developed by Key Solutions (http://keysolutions.ru/)
# All right reserved, 2006-2007
#######################################################################
"""CaptchaDemo class for the Zope 3 based ks.schema.captcha.demo package

$Id: captchademo.py 35330 2008-01-13 09:03:40Z cray $
"""

__author__  = "Anatoly Bubenkov"
__license__ = "ZPL"
__version__ = "$Revision: 35330 $"
__date__ = "$Date: 2008-01-13 12:03:40 +0300 (Вск, 13 Янв 2008) $"
__credits__ = "Based heavily on zope.app.form.browser.objectwidget.ObjectWidget"

from interfaces import ICaptchaDemo
from zope.schema.fieldproperty import FieldProperty
from zope.interface import implements
from persistent import Persistent
from zope.app.container.contained import Contained

class CaptchaDemo(Persistent, Contained):

    implements(ICaptchaDemo)



### -*- coding: utf-8 -*- #############################################
# Разработано компанией Ключевые Решения (http://keysolutions.ru/)
# Все права защищены, 2006-2007
#
# Developed by Key Solutions (http://keysolutions.ru/)
# All right reserved, 2006-2007
#######################################################################
"""Interfaces for the search object widget Zope 3 based package

$Id: calendarwidget.py 35330 2008-01-13 09:03:40Z cray $
"""
__author__  = "Anatoly Bubenkov"
__license__ = "ZPL"
__version__ = "$Revision: 35330 $"
__date__ = "$Date: 2008-01-13 12:03:40 +0300 (Вск, 13 Янв 2008) $"

from zope.interface import implements
from zope.app.form.browser import DateWidget
from zope.app.form.browser.widget import renderElement, renderTag
from zc.resourcelibrary import need
from zope.app.zapi import absoluteURL
from interfaces import ICalendarWidget

from ks.schema.date import _

SELECT_DATE_BUTTON_NAME = _(u'...')

RESOURCE_LIBRARY = 'ks.schema.date.browser.calendarwidget'

CALENDAR_SETUP = """Calendar.setup({
    inputField     :    "%(input_id)s",     // id of the input field
    displayArea      :    "%(displayArea)s",
    //button         :    "%(button_id)s",  // trigger for the calendar (button ID)
    align          :    "Bl",           // alignment (defaults to "Bl")
    singleClick    :    true,
    dateStatusFunc :    ourDateStatusFunc,
    date          :    "%(date)s",
    ifFormat      :    "%(format)s",
    daFormat      :    "%(displayformat)s",
    onSelect      :    onSelect,
});"""

CALENDAR_HTML = """<div id="calendar-top">
    <a href="#" class="next"><img src="%(resource_url)s/btn/next.gif" border="0" alt="" width="22" height="19" /></a>
    <a href="#" class="prev"><img src="%(resource_url)s/btn/prev.gif" border="0" alt="" width="22" height="19" /></a>
    <p class="date" id="%(displayArea)s">%(value)s</p>
    <div class="clear"></div>
</div>"""

import zope.component

class CalendarWidget(DateWidget):
    """Base Class For Search Object Fields"""

    implements(ICalendarWidget)

    @property
    def selectDateButtonName(self):
        return '%s-button' % self.name

    @property
    def buttonId(self):
        return '%s-button' % self.name

    @property
    def dateId(self):
        return '%s-date' % self.name

    def __call__(self):
        res = super(CalendarWidget, self).hidden()
        need(RESOURCE_LIBRARY)
        value = self._getFormValue()
        if value is None or value == self.context.missing_value:
            value = ''
        btn = CALENDAR_HTML % {'displayArea': self.dateId,
                               #'button_id': self.buttonId
                               'resource_url': '++resource++images',
                               'value': value
                               }

        src = renderElement('script',
                            type="text/javascript",
                            contents=CALENDAR_SETUP % {'input_id': self.name,
                                                       'displayArea': self.dateId,
                                                       'date': value,
                                                       'button_id': '',
                                                       'format': '%Y-%m-%d',
                                                       'displayformat': '%Y-%m-%d'})
        return "%s%s%s" % (res, btn, src)

    def hasInput(self):
        return ((self.name+".used" in self.request.form)
                or
                (self.name in self.request.form)
                )

class CalendarWidgetHidden(CalendarWidget):
    """Base Class For Search Object Fields"""

    def __call__(self):
        return self.hidden()



### -*- coding: utf-8 -*- #############################################
# Разработано компанией Ключевые Решения (http://keysolutions.ru/)
# Все права защищены, 2006-2007
#
# Developed by Key Solutions (http://keysolutions.ru/)
# All right reserved, 2006-2007
#######################################################################
"""The widget for repeated fields.

$Id: repeatwidget.py 35330 2008-01-13 09:03:40Z cray $
"""
__author__  = "Arvid"
__license__ = "ZPL"
__version__ = "$Revision: 35330 $"
__date__ = "$Date: 2008-01-13 12:03:40 +0300 (Вск, 13 Янв 2008) $"

from zope.interface import implements
from zope.schema.interfaces import ValidationError
from zope.app.form.interfaces import WidgetInputError
from zope.app.form.browser.widget import SimpleInputWidget
from zope.app.form.browser.textwidgets import TextWidget, PasswordWidget
from zope.schema.fieldproperty import FieldProperty

from interfaces import IRepeatWidget

from ks.schema.repeat import _

class NotRepeated(ValidationError):
    __doc__ = _(u"""Incorrect Repeat""")

class RepeatWidget(SimpleInputWidget):
    """base widget to repeat another fields"""
    implements(IRepeatWidget)

    checkfield = FieldProperty(IRepeatWidget['checkfield'])

    def getInputValue(self):
        value = super(RepeatWidget, self).getInputValue()

        try:
            if self.request.get(self.checkfield) != value:
                raise NotRepeated
        except NotRepeated, v:
            self._error = WidgetInputError(self.context.__name__, self.label, v)
            raise self._error

        return value

class PasswordRepeatWidget(PasswordWidget, RepeatWidget):
    """password repeat widget"""
    pass



### -*- coding: utf-8 -*- #############################################
# Разработано компанией Ключевые Решения (http://keysolutions.ru/)
# Все права защищены, 2006-2007
#
# Developed by Key Solutions (http://keysolutions.ru/)
# All right reserved, 2006-2007
#######################################################################
"""Simple Widget Factories module for the Zope 3 based ks.widget package

$Id: simplewidgetfactory.py 35330 2008-01-13 09:03:40Z cray $
"""

__author__  = "Anatoly Bubenkov"
__license__ = "ZPL"
__version__ = "$Revision: 35330 $"
__date__ = "$Date: 2008-01-13 12:03:40 +0300 (Вск, 13 Янв 2008) $"
__credits__ = "Based heavily on zope.app.form.browser.objectwidget.ObjectWidget"

import zope.component
from zope.app.form import CustomWidgetFactory
from zope.app.form.interfaces import IInputWidget, IDisplayWidget
import logging
logger = logging.getLogger('ks.widget.simplewidgetfactory')

def SchemaDisplayWidgetFactory(field, *args, **kw):
        # `field` is a bound field
        return CustomWidgetFactory(lambda *kv, **kw: zope.component.getMultiAdapter(
            (field, args[-1]), IDisplayWidget), *args, **kw)(field, args[-1])

def AddFormWidgetFactory(field, *args, **kw):
        # `field` is a bound field
        return CustomWidgetFactory(lambda *kv, **kw: zope.component.getMultiAdapter(
            (field, args[-1]), IInputWidget), *args, **kw)(field, args[-1])

def EditFormWidgetFactory(field, *args, **kw):
        # `field` is a bound field
        if field.readonly:
            iface = IDisplayWidget
        else:
            iface = IInputWidget
        return CustomWidgetFactory(lambda *kv, **kw: zope.component.getMultiAdapter(
            (field, args[-1]), iface), *args, **kw)(field, args[-1])



### -*- coding: utf-8 -*- #############################################
# Разработано компанией Ключевые Решения (http://keysolutions.ru/)
# Все права защищены, 2006-2007
#
# Developed by Key Solutions (http://keysolutions.ru/)
# All right reserved, 2006-2007
#######################################################################

from zope.app.form.browser import FileWidget
from xml.sax.saxutils import quoteattr, escape

import zope.i18nmessageid
_ = zope.i18nmessageid.MessageFactory('zope')

class DirectFileUploadWidget(FileWidget):
    """File Widget"""

    def _toFieldValue(self, input):
        if input is None or input == '':
            return self.context.missing_value
        try:
            seek = input.seek
            read = input.read
        except AttributeError, e:
            raise ConversionError(_('Form input is not a file object'), e)
        else:
            if read or getattr(input, 'filename', ''):
                return input
            else:
                return self.context.missing_value

    def _toFormValue(self, value):
        return 'File'

    def hasInput(self):
        return ((self.name+".used" in self.request.form)
                or
                (self.name in self.request.form)
                )



### -*- coding: utf-8 -*- #############################################
# Разработано компанией Ключевые Решения (http://keysolutions.ru/)
# Все права защищены, 2006-2007
#
# Developed by Key Solutions (http://keysolutions.ru/)
# All right reserved, 2006-2007
#######################################################################
"""HTMLDisplayWidget class for the Zope 3 based ks.widget package

$Id: htmldisplaywidget.py 35330 2008-01-13 09:03:40Z cray $
"""

__author__  = "Anatoly Bubenkov"
__license__ = "ZPL"
__version__ = "$Revision: 35330 $"
__date__ = "$Date: 2008-01-13 12:03:40 +0300 (Вск, 13 Янв 2008) $"
__credits__ = "Based heavily on zope.app.form.browser.objectwidget.ObjectWidget"

from zope.app.form.interfaces import IDisplayWidget
from zope.app.form.browser.widget import DisplayWidget, UnicodeDisplayWidget
from zope.interface import implements

from ks.schema.html import _

class UnicodeHTMLDisplayWidget(UnicodeDisplayWidget):

    implements (IDisplayWidget)

    def __call__(self):
        if self._renderedValueSet():
            value = self._data
        else:
            value = self.context.default
        if value == self.context.missing_value:
            return u""
        return unicode(value)

class HTMLDisplayWidget(DisplayWidget):

    implements (IDisplayWidget)

    def __call__(self):
        if self._renderedValueSet():
            value = self._data
        else:
            value = self.context.default
        if value == self.context.missing_value:
            return ""
        return value



### -*- coding: utf-8 -*- #############################################
# Разработано компанией Ключевые Решения (http://keysolutions.ru/)
# Все права защищены, 2006-2007
#
# Developed by Key Solutions (http://keysolutions.ru/)
# All right reserved, 2006-2007
#######################################################################

# Make it a Python package

import zope.i18nmessageid
_ = zope.i18nmessageid.MessageFactory('ks')

from object.objectdisplaywidget import ObjectDisplayWidget
from selectdate.selectdatewidget import SelectDateWidget, SelectDatetimeWidget
from html.htmldisplaywidget import UnicodeHTMLDisplayWidget, HTMLDisplayWidget
from html.htmlinputwidget import HTMLInputWidget
from smarturi.smarturiwidget import SmartURIWidget, SmartURIDisplayWidget
from captcha.captchawidget import CaptchaWidget
from image.imagedisplaywidget import ImageDisplayWidget
from repeat.repeatwidget import PasswordRepeatWidget
from re.rewidget import EmailWidget, NameWidget, WidgetREBase
from externalurl.externalurlwidget import ExternalURLWidget, ExternalURLsWidget
from ks.smartimage.smartimageschema.smartimagewidget import SmartImageWidget
from ks.smartimage.smartimageschema.smartimagedisplaywidget import SmartImageDisplayWidget
from directfileupload.directfileuploadwidget import DirectFileUploadWidget



### -*- coding: utf-8 -*- #############################################
# Разработано компанией Ключевые Решения (http://keysolutions.ru/)
# Все права защищены, 2006-2007
#
# Developed by Key Solutions (http://keysolutions.ru/)
# All right reserved, 2006-2007
#######################################################################
"""CaptchaWidget class for the Zope 3 based ks.widget package

$Id: captchawidget.py 35330 2008-01-13 09:03:40Z cray $
"""

__author__  = "Anatoly Bubenkov"
__license__ = "ZPL"
__version__ = "$Revision: 35330 $"
__date__ = "$Date: 2008-01-13 12:03:40 +0300 (Вск, 13 Янв 2008) $"

import datetime

from zope.app.form.interfaces import ConversionError, InputErrors
from zope.app.form.browser.widget import renderElement
from zope.app.form.browser import IntWidget
from ks.lib.pagetemplate.viewpagetemplatestring import ViewPageTemplateString

from ks.schema.captcha import _

class CaptchaWidget(IntWidget):

    imageCssClass = u'captcha'

    noImageTemplate = u''

    @property
    def keyName(self):
        return '%s.key' % self.name

    def __call__(self):
        res = []
        key = self.context.captcha.getkey()
        kwargs = {'type': 'hidden',
                  'name': self.keyName,
                  'id': self.keyName,
                  'value': key}
        res.append(renderElement('input', **kwargs))
        kwargs = {'src': '@@captcha?key=%s' % key,
                  'alt': _('Turn on images'),
                  'cssClass': self.imageCssClass
                  }
        res.append(renderElement('img', **kwargs))
        res.append(super(CaptchaWidget, self).__call__())
        pt = ViewPageTemplateString(source=self.noImageTemplate)
        res.append(pt(self, self.context.context, self.request))
        return '\n'.join(res)

    def _toFieldValue(self, input):
        if self.convert_missing_value and input == self._missing:
            value = self.context.missing_value
        else:
            try:
                value = map(unicode, input)
            except ValueError, v:
                raise ConversionError(_('Incorrect text data'), v)
            if not value[0].isdigit():
                raise ConversionError(_(u'Random key must consist of digits!'))
            if not value[1] and not self.context.required:
                value[1] = None
                return value
            if not value[1].isdigit():
                raise ConversionError(_(u'Code must consist of digits!'))
            try:
                value = map(int, input)
            except ValueError, v:
                raise ConversionError(_(u'Data conversion error'), v)

        return value

    def _getFormInput(self):
        return (self.request.get(self.keyName), self.request.get(self.name))

    def _toFormValue(self, value):
        return self._missing



### -*- coding: utf-8 -*- #############################################
# Разработано компанией Ключевые Решения (http://keysolutions.ru/)
# Все права защищены, 2006-2007
#
# Developed by Key Solutions (http://keysolutions.ru/)
# All right reserved, 2006-2007
#######################################################################
"""HTMLInputWidget class for the Zope 3 based ks.widget package

$Id: htmlinputwidget.py 35330 2008-01-13 09:03:40Z cray $
"""

__author__  = "Anatoly Zaretsky"
__license__ = "ZPL"
__version__ = "$Revision: 35330 $"
__date__ = "$Date: 2008-01-13 12:03:40 +0300 (Вск, 13 Янв 2008) $"

import json

from zope.component import queryMultiAdapter
from zope.app.container.interfaces import IAdding

import z3c.widget.tiny.widget

from interfaces import IHTMLImageListProvider

from ks.schema.html import _

class HTMLInputWidget(z3c.widget.tiny.widget.TinyWidget):

    height = 30
    mce_theme = "advanced"
    mce_theme_advanced_toolbar_location = "top"
    mce_theme_advanced_toolbar_align = "left"
    mce_theme_advanced_statusbar_location = "bottom"
    mce_entity_encoding = "raw"
    mce_convert_newlines_to_brs = "true"
    mce_external_image_list_url = "htmlimagelist.js"
    mce_relative_urls = "false"
    mce_plugins = "style,layer,table,save,advhr,advimage,advlink,emotions,iespell,insertdatetime,preview,zoom,media,searchreplace,print,contextmenu,paste,directionality,fullscreen,noneditable,visualchars,nonbreaking,xhtmlxtras"
    mce_theme_advanced_buttons1 = "save,newdocument,|,bold,italic,underline,strikethrough,|,justifyleft,justifycenter,justifyright,justifyfull,|,styleselect,formatselect,fontselect,fontsizeselect"
    mce_theme_advanced_buttons2 = "cut,copy,paste,pastetext,pasteword,|,search,replace,|,bullist,numlist,|,outdent,indent,|,undo,redo,|,link,unlink,anchor,image,cleanup,code,|,insertdate,inserttime,preview,|,forecolor,backcolor"
    mce_theme_advanced_buttons3 = "tablecontrols,|,hr,removeformat,visualaid,|,sub,sup,|,charmap,emotions,iespell,media,advhr,|,print,|,ltr,rtl,|,fullscreen"
    mce_theme_advanced_buttons4 = "insertlayer,moveforward,movebackward,absolute,|,styleprops,|,cite,abbr,acronym,del,ins,attribs,|,visualchars,nonbreaking"
    mce_extended_valid_elements = "a[name|href|target|title|onclick],img[class|src|border=0|alt|title|hspace|vspace|width|height|align|onmouseover|onmouseout|name],hr[class|width|size|noshade],font[face|size|color|style],span[class|align|style]"


class HTMLImageListJS(object):

    def __call__(self, *kv, **kw):
        context = self.context
        if IAdding.providedBy(context):
            context = context.context
        images = getattr(
            queryMultiAdapter(
                (context, self.request),
                interface=IHTMLImageListProvider,
                context=context),
            'images',
            None)
        self.request.response.setHeader('Content-Type', 'text/javascript; charset=utf-8')
        if images:
            return u'var tinyMCEImageList = %s;\n' % json.write(images)
        return u''



### -*- coding: utf-8 -*- #############################################
# Разработано компанией Ключевые Решения (http://keysolutions.ru/)
# Все права защищены, 2006-2007
#
# Developed by Key Solutions (http://keysolutions.ru/)
# All right reserved, 2006-2007
#######################################################################
"""Image field class for the Zope 3 based ks.schema.image package

$Id: image.py 35330 2008-01-13 09:03:40Z cray $
"""

__author__  = "Anatoly Zaretsky"
__license__ = "ZPL"
__version__ = "$Revision: 35330 $"
__date__ = "$Date: 2008-01-13 12:03:40 +0300 (Вск, 13 Янв 2008) $"

from zope.schema import Bytes
from zope.interface import implements

from interfaces import IImage


class Image(Bytes):
    implements(IImage)



### -*- coding: utf-8 -*- #############################################
# Разработано компанией Ключевые Решения (http://keysolutions.ru/)
# Все права защищены, 2006-2007
#
# Developed by Key Solutions (http://keysolutions.ru/)
# All right reserved, 2006-2007
#######################################################################
"""Inrefaces for the Zope 3 based ks.schema.captcha package

$Id: interfaces.py 35330 2008-01-13 09:03:40Z cray $
"""
__author__  = "Anatoly Bubenkov"
__license__ = "ZPL"
__version__ = "$Revision: 35330 $"
__date__ = "$Date: 2008-01-13 12:03:40 +0300 (Вск, 13 Янв 2008) $"

from zope.schema.interfaces import IField
from zope.schema import TextLine, ASCIILine, ValidationError
from zope.interface import Interface, Attribute
from zope.schema.interfaces import ValidationError
from ks.schema.captcha import _

class CodeIsInvalidError(ValidationError):
    __doc__ = _(u'Codes is not equal!')

class ICaptcha(IField):
    """Captcha interface"""

    captcha = Attribute("captcha",
                             ("ks.captcha.interfaces.ICaptcha to be used, or image checker utility name, or None.\n"
                             "\n"
                             "If a string, the image checker name should be used by an\n"
                             "registered ks.captcha.interfaces.ICaptcha utility name"))



### -*- coding: utf-8 -*- #############################################
# Разработано компанией Ключевые Решения (http://keysolutions.ru/)
# Все права защищены, 2006-2007
#
# Developed by Key Solutions (http://keysolutions.ru/)
# All right reserved, 2006-2007
#######################################################################
"""HMTMLDisplayWidget class for the Zope 3 based ks.widget package

$Id: smarturi.py 35330 2008-01-13 09:03:40Z cray $
"""
__author__  = "Anatoly Bubenkov"
__license__ = "ZPL"
__version__ = "$Revision: 35330 $"
__date__ = "$Date: 2008-01-13 12:03:40 +0300 (Вск, 13 Янв 2008) $"

from zope.schema import Object
from interfaces import ISmartURI, IURI
from zope.interface import implements
from zope.interface.interfaces import IInterface

class URI(object):

    implements(IURI)

    title = IURI['title'].default

    uri = IURI['uri'].default

    def __init__(self, title=None, uri=None):
        self.title = title
        self.uri = uri

class SmartURI(Object):

    implements(ISmartURI)

    def __init__(self, **kw):
        schema = kw.get('schema', None)
        if not IInterface.providedBy(schema):
            schema = IURI
        super(SmartURI, self).__init__(schema, **kw)



### -*- coding: utf-8 -*- #############################################
# Разработано компанией Ключевые Решения (http://keysolutions.ru/)
# Все права защищены, 2006-2007
#
# Developed by Key Solutions (http://keysolutions.ru/)
# All right reserved, 2006-2007
#######################################################################
"""ImageDisplayWidget class for the Zope 3 based ks.widget package

$Id: imagedisplaywidget.py 35330 2008-01-13 09:03:40Z cray $
"""

__author__  = "Anatoly Zaretsky"
__license__ = "ZPL"
__version__ = "$Revision: 35330 $"
__date__ = "$Date: 2008-01-13 12:03:40 +0300 (Вск, 13 Янв 2008) $"

from zope.interface import implements
from zope.traversing.browser import absoluteURL

from zope.app.form.interfaces import IDisplayWidget
from zope.app.form.browser.widget import DisplayWidget, renderElement


# from ks.widget import _ не используется

class ImageDisplayWidget(DisplayWidget):

    implements (IDisplayWidget)

    cssClass = u''
    extra = u''

    def __call__(self):
        src = '%s/++attribute++%s/++attribute++__call__' % (absoluteURL(self.context.context, self.request), self.context.__name__)
        return renderElement('img', src=src, cssClass=self.cssClass, extra=self.extra)



### -*- coding: utf-8 -*- #############################################
# Разработано компанией Ключевые Решения (http://keysolutions.ru/)
# Все права защищены, 2006-2007
#
# Developed by Key Solutions (http://keysolutions.ru/)
# All right reserved, 2006-2007
#######################################################################
"""HMTMLDisplayWidget class for the Zope 3 based ks.widget package

$Id: smarturiwidget.py 35330 2008-01-13 09:03:40Z cray $
"""

__author__  = "Anatoly Bubenkov"
__license__ = "ZPL"
__version__ = "$Revision: 35330 $"
__date__ = "$Date: 2008-01-13 12:03:40 +0300 (Вск, 13 Янв 2008) $"
__credits__ = "Based heavily on zope.app.form.browser.objectwidget.ObjectWidget"

from ks.schema.smarturi.smarturi import URI

from zope.app.form import CustomWidgetFactory
from zope.app.form.browser import ObjectWidget, DisplayWidget
from zope.app.form.browser.widget import renderElement
from zope.interface import implements
from interfaces import ISmartURIDisplayWidget

from interfaces import _

SmartURIWidget = CustomWidgetFactory(ObjectWidget, URI)

class SmartURIDisplayWidget(DisplayWidget):

    implements(ISmartURIDisplayWidget)

    tag = 'a'

    cssClass = u''

    extra = u''

    _missing = u''

    def __call__(self):
        if self._renderedValueSet():
            value = self._data
        else:
            value = self.context.default
        if value == self.context.missing_value:
            value = self._missing
        return renderElement(self.tag,
                             id=self.name,
                             contents=value.title,
                             href=value.uri,
                             cssClass=self.cssClass,
                             extra=self.extra)
